@@ -56,6 +56,15 @@ gtk_html_stream_destroy (GtkHTMLStream *stream)
 	g_free (stream);
 }
 
+/**
+ * gtk_html_stream_write:
+ * @stream:
+ * @buffer:
+ * @size:
+ *
+ * Write data to a GtkHTMLStream.
+ *
+ */
 void
 gtk_html_stream_write (GtkHTMLStream *stream,
 		       const gchar *buffer,
@@ -74,12 +83,17 @@ gtk_html_stream_vprintf (GtkHTMLStream *stream,
 			 const char *format,
 			 va_list ap)
 {
-	size_t len = g_printf_string_upper_bound (format, ap);
-	char *buf;
+	size_t len;
+	char *buf = NULL;
 	char *mbuf = NULL;
 	int rv;
+	va_list ap_copy;
 
-	buf = alloca (len);
+	G_VA_COPY (ap_copy, ap);
+	len = g_printf_string_upper_bound (format, ap_copy);
+
+	if (len < 8192)
+		buf = alloca (len);
 
 	if (buf == NULL)
 		buf = mbuf = g_malloc (len);
